@@ -7,14 +7,14 @@ use UR;
 use Data::Dumper;
 use File::Basename;
 use Getopt::Long;
-use Term::ANSIColor;
+use Term::ANSIColor qw();
 require Text::Wrap;
 
-our $VERSION = "0.41"; # UR $VERSION;
+our $VERSION = "0.46"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
-    is => 'Command',
+    is => ['Command', 'Command::Common'],
     is_abstract => 1,
     attributes_have => [
         is_input            => { is => 'Boolean', is_optional => 1 },
@@ -24,6 +24,7 @@ UR::Object::Type->define(
                                  doc => 'when set, this property is a positional argument when run from a shell' },
     ],
     has_optional => [
+        debug       => { is => 'Boolean', doc => 'enable debug messages' },
         is_executed => { is => 'Boolean' },
         result      => { is => 'Scalar', is_output => 1 },
         original_command_line => { is => 'String', doc => 'null-byte separated list of command and arguments when run via execute_with_shell_params_and_exit'},
@@ -39,22 +40,6 @@ eval {
     binmode STDERR, ":utf8";
 };
 
-# Override method in UR::Object to support error_die and error_rv_false
-sub validate_subscription {
-    my $self = shift;
-    my $subscription_property = shift;
-
-    my $retval = $self->SUPER::validate_subscription($subscription_property, @_);
-    return $retval if $retval;
-
-    unless ( defined($subscription_property) and $subscription_property eq 'error_die') {
-        $subscription_property = '(undef)' unless defined ($subscription_property);
-        Carp::croak("Unrecognized subscription aspect '$subscription_property'");
-    }
-
-    return 1;
-}
-
 sub _init_subclass {
     # Each Command subclass has an automatic wrapper around execute().
     # This ensures it can be called as a class or instance method,
@@ -83,87 +68,6 @@ sub _init_subclass {
     return 1;
 }
 
-sub shortcut {
-    my $self = shift;
-    return unless $self->can('_shortcut_body');
-
-    my $result = $self->_shortcut_body;
-    $self->result($result);
-
-    return $result;
-}
-
-sub execute {
-    # This is a wrapper for real execute() calls.
-    # All execute() methods are turned into _execute_body at class init,
-    # so this will get direct control when execute() is called.
-    my $self = shift;
-
-    #TODO handle calls to SUPER::execute() from another execute().
-
-    # handle calls as a class method
-    my $was_called_as_class_method = 0;
-    if (ref($self)) {
-        if ($self->is_executed) {
-            Carp::confess("Attempt to re-execute an already executed command.");
-        }
-    }
-    else {
-        # called as class method
-        # auto-create an instance and execute it
-        $self = $self->create(@_);
-        return unless $self;
-        $was_called_as_class_method = 1;
-    }
-
-    # handle __errors__ objects before execute
-    if (my @problems = $self->__errors__) {
-        for my $problem (@problems) {
-            my @properties = $problem->properties;
-            $self->error_message("Property " .
-                                 join(',', map { "'$_'" } @properties) .
-                                 ': ' . $problem->desc);
-        }
-        my $command_name = $self->command_name;
-        $self->error_message("Please see '$command_name --help' for more information.");
-        $self->delete() if $was_called_as_class_method;
-        return;
-    }
-
-    my $result;
-    eval { $result = $self->_execute_body(@_); };
-    my $error = $@;
-    if ($error or not $result) {
-        my %error_data;
-
-        $error_data{die_message} = defined($error) ? $error:'';
-        $error_data{error_message} = defined($self->error_message) ? $self->error_message:'';
-        $error_data{error_package} = defined($self->error_package) ? $self->error_package:'';
-        $error_data{error_file} = defined($self->error_file) ? $self->error_file:'';
-        $error_data{error_subroutine} = defined($self->error_subroutine) ? $self->error_subroutine:'';
-        $error_data{error_line} = defined($self->error_line) ? $self->error_line:'';
-        $self->__signal_observers__('error_die', %error_data);
-        die $error if $error;
-    }
-
-    $self->is_executed(1);
-    $self->result($result);
-
-    return $self if $was_called_as_class_method;
-    return $result;
-}
-
-sub _execute_body {
-    # default implementation in the base class
-    my $self = shift;
-    my $class = ref($self) || $self;
-    if ($class eq __PACKAGE__) {
-        die "The execute() method is not defined for $_[0]!";
-    }
-    return 1;
-}
-
-
 #
 # Standard external interface for shell dispatchers
 #
@@ -274,7 +178,10 @@ sub _execute_delegate_class_with_params {
     $command_object->dump_status_messages(1);
     $command_object->dump_warning_messages(1);
     $command_object->dump_error_messages(1);
-    $command_object->dump_debug_messages(0);
+    $command_object->dump_debug_messages($command_object->debug);
+    if ($command_object->debug) {
+        UR::ModuleBase->dump_debug_messages($command_object->debug);
+    }
 
     my $rv = $command_object->execute($params);
 
@@ -285,30 +192,6 @@ sub _execute_delegate_class_with_params {
     return $rv;
 }
 
-#
-# Standard programmatic interface
-#
-
-sub create {
-    my $class = shift;
-    my ($rule,%extra) = $class->define_boolexpr(@_);
-    my @params_list = $rule->params_list;
-    my $self = $class->SUPER::create(@params_list, %extra);
-    return unless $self;
-
-    # set non-optional boolean flags to false.
-    for my $property_meta ($self->_shell_args_property_meta) {
-        my $property_name = $property_meta->property_name;
-        if (!$property_meta->is_optional and !defined($self->$property_name)) {
-            if (defined $property_meta->data_type and $property_meta->data_type =~ /Boolean/i) {
-                $self->$property_name(0);
-            }
-        }
-    }
-
-    return $self;
-}
-
 #
 # Methods to override in concrete subclasses.
 #
@@ -328,24 +211,6 @@ sub _bare_shell_argument_names {
     return @ordered_names;
 }
 
-# Translates a true/false value from the command module's execute()
-# from Perl (where positive means success), to shell (where 0 means success)
-# Also, execute() could return a negative value; this is converted to
-# positive and used as the shell exit code.  NOTE: This means execute()
-# returning 0 and -1 mean the same thing
-sub exit_code_for_return_value {
-    my $self = shift;
-    my $return_value = shift;
-    if (! $return_value) {
-        $return_value = 1;
-    } elsif ($return_value < 0) {
-        $return_value = 0 - $return_value;
-    } else {
-        $return_value = 0
-    }
-    return $return_value;
-}
-
 sub help_brief {
     my $self = shift;
     if (my $doc = $self->__meta__->doc) {
@@ -463,7 +328,6 @@ sub command_name {
     my $self = shift;
     my $class = ref($self) || $self;
     my $prepend = '';
-    $DB::single = 1;
     if (defined($Command::entry_point_class) and $class =~ /^($Command::entry_point_class)(::.+|)$/) {
         $prepend = $Command::entry_point_bin;
         $class = $2;
@@ -908,6 +772,7 @@ sub help_options {
         #$param_name = "--$param_name";
         my $doc = $property_meta->doc;
         my $valid_values = $property_meta->valid_values;
+        my $example_values = $property_meta->example_values;
         unless ($doc) {
             # Maybe a parent class has documentation for this property
             eval {
@@ -936,6 +801,13 @@ sub help_options {
             }
             chomp $doc;
         }
+        if ($example_values && @$example_values) {
+            $doc .= "\nexample" . (@$example_values > 1 and 's') . ":\n";
+            $doc .= join(', ',
+                        map { ref($_) ? Data::Dumper->new([$_])->Terse(1)->Dump() : $_ } @$example_values
+                    );
+            chomp($doc);
+        }
         $max_name_length = length($param_name) if $max_name_length < length($param_name);
 
         my $param_type = $property_meta->data_type || '';
@@ -943,8 +815,14 @@ sub help_options {
             $param_type = ucfirst(lc($param_type));
         }
 
-        my $default_value = $property_meta->default_value;
-        if (defined $default_value) {
+        my $default_value;
+        if (defined($default_value = $property_meta->default_value)
+            || defined(my $calculated_default = $property_meta->calculated_default)
+        ) {
+            unless (defined $default_value) {
+                $default_value = $calculated_default->()
+            }
+
             if ($param_type eq 'Boolean') {
                 $default_value = $default_value ? "'true'" : "'false' (--no$param_name)";
             } elsif ($property_meta->is_many && ref($default_value) eq 'ARRAY') {
@@ -1132,7 +1010,6 @@ sub help_sub_commands {
         }
         $text .= "\n";
     }
-    #$DB::single = 1;
     return $text;
 }
 
