@@ -11,14 +11,14 @@ use Getopt::Long;
 use Command::View::DocMethods;
 use Command::Dispatch::Shell;
 
-our $VERSION = "0.40"; # UR $VERSION;
+our $VERSION = "0.46"; # UR $VERSION;
 
 our $entry_point_class;
 our $entry_point_bin;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
-    is => 'Command',
+    is => ['Command', 'Command::Common'],
     is_abstract => 1,
     subclass_description_preprocessor => 'Command::V2::_preprocess_subclass_description',
     attributes_have => [
@@ -33,6 +33,7 @@ UR::Object::Type->define(
                                 doc => 'when expanding user supplied values: 0 = never verify, 1 = always verify, undef = determine automatically', },        
     ],
     has_optional => [
+        debug       => { is => 'Boolean', doc => 'enable debug messages' },
         is_executed => { is => 'Boolean' },
         result      => { is => 'Scalar', is_output => 1 },
         original_command_line => { is => 'String', doc => 'null-byte separated list of command and arguments when run via execute_with_shell_params_and_exit'},
@@ -112,27 +113,6 @@ sub _init_subclass {
     return 1;
 }
 
-sub create {
-    my $class = shift;
-    my ($rule,%extra) = $class->define_boolexpr(@_);
-    my @params_list = $rule->params_list;
-    my $self = $class->SUPER::create(@params_list, %extra);
-    return unless $self;
-
-    # set non-optional boolean flags to false.
-    # TODO: rename that property meta method if it is not ONLY used for shell args
-    for my $property_meta ($self->_shell_args_property_meta) {
-        my $property_name = $property_meta->property_name;
-        if (!$property_meta->is_optional and !defined($self->$property_name)) {
-            if (defined $property_meta->data_type and $property_meta->data_type =~ /Boolean/i) {
-                $self->$property_name(0);
-            }
-        }
-    }    
-    
-    return $self;
-}
-
 sub __errors__ {
     my ($self,@property_names) = @_;
     my @errors1 =($self->SUPER::__errors__);
@@ -165,96 +145,49 @@ sub is_sub_command_delegator {
     return;
 }
 
-sub shortcut {
-    my $self = shift;
-    return unless $self->can('_shortcut_body');
+sub _wrapper_has {
+    my ($class, $new_class_base) = @_;
 
-    my $result = $self->_shortcut_body;
-    $self->result($result);
+    $new_class_base ||= __PACKAGE__;
 
-    return $result;
-}
+    my $command_meta = $class->__meta__;
+    my @properties = $command_meta->properties();
+    
+    my %has;
+    for my $property (@properties) {
+        my %desc;
+        next unless $property->can("is_param") and $property->can("is_input") and $property->can("is_output");
 
-sub execute {
-    # This is a wrapper for real execute() calls.
-    # All execute() methods are turned into _execute_body at class init, 
-    # so this will get direct control when execute() is called. 
-    my $self = shift;
+        my $name = $property->property_name;
 
-    #TODO handle calls to SUPER::execute() from another execute().    
+        next if $new_class_base->can($name);
 
-    # handle calls as a class method
-    my $was_called_as_class_method = 0;
-    if (ref($self)) {
-        if ($self->is_executed) {
-            Carp::confess("Attempt to re-execute an already executed command.");
+        if ($property->is_param) {
+            $desc{is_param} = 1;
         }
-    }
-    else {
-        # called as class method
-        # auto-create an instance and execute it
-        $self = $self->create(@_);
-        return unless $self;
-        $was_called_as_class_method = 1;
-    }
-
-    # handle __errors__ objects before execute
-    if (my @problems = $self->__errors__) {
-        for my $problem (@problems) {
-            my @properties = $problem->properties;
-            $self->error_message("Property " .
-                                 join(',', map { "'$_'" } @properties) .
-                                 ': ' . $problem->desc);
+        elsif ($property->is_input) {
+            $desc{is_input} = 1;
         }
-        $self->delete() if $was_called_as_class_method;
-        return;
+        #elsif ($property->can("is_metric") and $property->is_metric) {
+        #    $desc{is_metric} = 1;
+        #}
+        #elsif ($property->can("is_output") and $property->is_output) {
+        #    $desc{is_output} = 1;
+        #}
+        else {
+            next;
+        }
+
+        $has{$name} = \%desc;
+        $desc{is} = $property->data_type;
+        $desc{doc} = $property->doc;
+        $desc{is_many} = $property->is_many;
+        $desc{is_optional} = $property->is_optional;
     }
 
-    my $result = $self->_execute_body(@_);
-
-    $self->is_executed(1);
-    $self->result($result);
-
-    return $self if $was_called_as_class_method;
-    return $result;
+    return %has;
 }
 
-sub _execute_body {    
-    # default implementation in the base class
-
-    # Override "execute" or "_execute_body" to implement the body of the command.
-    # See above for details of internal implementation.
-
-    my $self = shift;
-    my $class = ref($self) || $self;
-    if ($class eq __PACKAGE__) {
-        die "The execute() method is not defined for $_[0]!";
-    }
-    return 1;
-}
-
-
-sub exit_code_for_return_value {
-    my $self = shift;
-    my $return_value = shift;
-
-    # Translates a true/false value from the command module's execute()
-    # from Perl (where positive means success), to shell (where 0 means success)
-    # Also, execute() could return a negative value; this is converted to
-    # positive and used as the shell exit code.  NOTE: This means execute()
-    # returning 0 and -1 mean the same thing
-
-    if (! $return_value) {
-        $return_value = 1;
-    } elsif ($return_value < 0) {
-        $return_value = 0 - $return_value;
-    } else {
-        $return_value = 0 
-    }
-    return $return_value;
-}
-
-
 sub display_command_summary_report {
     my $self = shift;
     my $total_count = $self->_total_command_count;
