@@ -2,7 +2,7 @@ package Command::V2;  # additional methods to produce documentation, TODO: turn
 use strict;
 use warnings;
 
-use Term::ANSIColor;
+use Term::ANSIColor qw();
 use Pod::Simple::Text;
 require Text::Wrap;
 
@@ -81,35 +81,70 @@ sub doc_help {
 
     # standard: update this to do the old --help format
     my $synopsis = $self->help_synopsis;
-    my $required_args = $self->help_options(is_optional => 0);
-    my $optional_args = $self->help_options(is_optional => 1);
-    $text = sprintf(
-        "\n%s\n%s\n\n%s%s%s%s%s\n",
-        Term::ANSIColor::colored('USAGE', 'underline'),
+    my $required_inputs = $self->help_options(is_optional => 0, is_input => 1);
+    my $required_outputs = $self->help_options(is_optional => 0, is_output => 1);
+    my $required_params = $self->help_options(is_optional => 0, is_param => 1);
+    my $optional_inputs = $self->help_options(is_optional => 1, is_input => 1);
+    my $optional_outputs = $self->help_options(is_optional => 1, is_output => 1);
+    my $optional_params = $self->help_options(is_optional => 1, is_param => 1);
+    my @parts;
+    
+    push @parts, Term::ANSIColor::colored('USAGE', 'underline');
+    push @parts, 
         Text::Wrap::wrap(
             ' ', 
             '    ', 
             Term::ANSIColor::colored($self->command_name, 'bold'),
             $self->_shell_args_usage_string || '',
-        ),
+        );
+
+    push @parts, 
         ( $synopsis 
             ? sprintf("%s\n%s\n", Term::ANSIColor::colored("SYNOPSIS", 'underline'), $synopsis)
             : ''
-        ),
-        ( $required_args 
-            ? sprintf("%s\n%s\n", Term::ANSIColor::colored("REQUIRED ARGUMENTS", 'underline'), $required_args)
+        );
+    push @parts, 
+        ( $required_inputs
+            ? sprintf("%s\n%s\n", Term::ANSIColor::colored("REQUIRED INPUTS", 'underline'), $required_inputs)
             : ''
-        ),
-        ( $optional_args 
-            ? sprintf("%s\n%s\n", Term::ANSIColor::colored("OPTIONAL ARGUMENTS", 'underline'), $optional_args)
+        );
+    push @parts, 
+        ( $required_params
+            ? sprintf("%s\n%s\n", Term::ANSIColor::colored("REQUIRED PARAMS", 'underline'), $required_params)
             : ''
-        ),
+        );
+    push @parts, 
+        ( $optional_inputs
+            ? sprintf("%s\n%s\n", Term::ANSIColor::colored("OPTIONAL INPUTS", 'underline'), $optional_inputs)
+            : ''
+        );
+    push @parts, 
+        ( $optional_params
+            ? sprintf("%s\n%s\n", Term::ANSIColor::colored("OPTIONAL PARAMS", 'underline'), $optional_params)
+            : ''
+        );
+    push @parts, 
+        ( $required_outputs
+            ? sprintf("%s\n%s\n", Term::ANSIColor::colored("REQUIRED OUTPUTS", 'underline'), $required_outputs)
+            : ''
+        );
+    push @parts, 
+        ( $optional_outputs
+            ? sprintf("%s\n%s\n", Term::ANSIColor::colored("OPTIONAL OUTPUTS", 'underline'), $optional_outputs)
+            : ''
+        );
+    push @parts, 
         sprintf(
             "%s\n%s\n",
             Term::ANSIColor::colored("DESCRIPTION", 'underline'),
             _pod2txt($self->help_detail || '')
-        ),
-        ( $extra_help ? $extra_help : '' ),
+        );
+    push @parts, 
+        ( $extra_help ? $extra_help : '' );
+
+    $text = sprintf(
+        "\n%s\n%s\n\n%s%s%s%s%s%s%s%s%s\n",
+        @parts
     );
 
     return $text;
@@ -356,6 +391,7 @@ sub help_options {
         #$param_name = "--$param_name";
         my $doc = $property_meta->doc;
         my $valid_values = $property_meta->valid_values;
+        my $example_values = $property_meta->example_values;
         unless ($doc) {
             # Maybe a parent class has documentation for this property
             eval {
@@ -384,6 +420,13 @@ sub help_options {
             }
             chomp $doc;
         }
+        if ($example_values && @$example_values) {
+            $doc .= "\nexample" . (@$example_values > 1 and 's') . ":\n";
+            $doc .= join(', ',
+                        map { ref($_) ? Data::Dumper->new([$_])->Terse(1)->Dump() : $_ } @$example_values
+                    );
+            chomp($doc);
+        }
         $max_name_length = length($param_name) if $max_name_length < length($param_name);
 
         my $param_type = $property_meta->data_type || '';
@@ -391,8 +434,14 @@ sub help_options {
             $param_type = ucfirst(lc($param_type));
         }
 
-        my $default_value = $property_meta->default_value;
-        if (defined $default_value) {
+        my $default_value;
+        if (defined($default_value = $property_meta->default_value)
+            || defined(my $calculated_default = $property_meta->calculated_default)
+        ) {
+            unless (defined $default_value) {
+                $default_value = $calculated_default->()
+            }
+
             if ($param_type eq 'Boolean') {
                 $default_value = $default_value ? "'true'" : "'false' (--no$param_name)";
             } elsif ($property_meta->is_many && ref($default_value) eq 'ARRAY') {
@@ -495,30 +544,26 @@ sub _doc_see_also {
 sub _shell_args_usage_string {
     my $self = shift;
 
-    my $is_executable = eval {
-        if ($self->can("_execute_body") eq __PACKAGE__->can("_execute_body")) {
-            return;
+    return eval {
+        if ( $self->isa('Command::Tree') ) { 
+            return '...';
+        }
+        elsif ($self->can("_execute_body") eq __PACKAGE__->can("_execute_body")) {
+            return '(no execute!)';
         }
         elsif ($self->__meta__->is_abstract) {
-            return;
+            return '(no sub commands!)';
         }
         else {
-            return 1;
+            return join(
+                " ", 
+                map { 
+                    $self->_shell_arg_usage_string_from_property_meta($_) 
+                } $self->_shell_args_property_meta()
+
+            );
         }
     };
-
-    if ($is_executable) {
-        return join(
-            " ", 
-            map { 
-                $self->_shell_arg_usage_string_from_property_meta($_) 
-            } $self->_shell_args_property_meta()
-            
-        );
-    }
-    else {
-        return "(no execute or sub commands implemented)"
-    }
 }
 
 sub _shell_args_usage_string_abbreviated {
@@ -532,11 +577,39 @@ sub _shell_args_usage_string_abbreviated {
     }
 }
 
+sub sub_command_mapping {
+    my ($self, $class) = @_;
+    return if !$class;
+    no strict 'refs';
+    my $mapping = ${ $class . '::SUB_COMMAND_MAPPING'};
+    if (ref($mapping) eq 'HASH') {
+        return $mapping;
+    } else {
+        return;
+    }
+};
+
 sub command_name {
     my $self = shift;
     my $class = ref($self) || $self;
     my $prepend = '';
-    if (defined($Command::entry_point_class) and $class =~ /^($Command::entry_point_class)(::.+|)$/) {
+
+
+    # There can be a hash in the command entry point class that maps
+    # root level tools to classes so they can be in a different location
+    # ...this bit of code considers that misdirection:
+    my $entry_point_class = $Command::entry_point_class;
+    my $mapping = $self->sub_command_mapping($entry_point_class);
+    for my $k (%$mapping) {
+        my $v = $mapping->{$k};
+        if ($v && $v eq $class) {
+            my @words = grep { $_ ne 'Command' } split(/::/,$class);
+            return join(' ', $self->_command_name_for_class_word($words[0]), $k);
+        }
+    }
+
+
+    if (defined($entry_point_class) and $class =~ /^($entry_point_class)(::.+|)$/) {
         $prepend = $Command::entry_point_bin;
         $class = $2;
         if ($class =~ s/^:://) {
