@@ -3,7 +3,7 @@ package UR::All;
 use strict;
 use warnings;
 
-our $VERSION = "0.40"; # UR $VERSION;
+our $VERSION = "0.46"; # UR $VERSION;
 
 BEGIN { require above; };
 use UR;
@@ -35,7 +35,7 @@ use UR::BoolExpr::Template::PropertyComparison::LessOrEqual;
 use UR::BoolExpr::Template::PropertyComparison::LessThan;
 use UR::BoolExpr::Template::PropertyComparison::Like;
 use UR::BoolExpr::Template::PropertyComparison::Matches;
-use UR::BoolExpr::Template::PropertyComparison::NotEqual;
+use UR::BoolExpr::Template::PropertyComparison::NotEquals;
 use UR::BoolExpr::Template::PropertyComparison::NotIn;
 use UR::BoolExpr::Template::PropertyComparison::NotLike;
 use UR::BoolExpr::Template::PropertyComparison::True;
@@ -103,7 +103,6 @@ use UR::Env::UR_IGNORE;
 use UR::Env::UR_NR_CPU;
 use UR::Env::UR_STACK_DUMP_ON_DIE;
 use UR::Env::UR_STACK_DUMP_ON_WARN;
-use UR::Env::UR_TEST_FILLDB;
 use UR::Env::UR_TEST_QUIET;
 use UR::Env::UR_USE_ANY;
 use UR::Env::UR_USE_DUMMY_AUTOGENERATED_IDS;
@@ -205,7 +204,6 @@ use UR::Service::RPC::Message;
 use UR::Service::RPC::Server;
 use UR::Service::RPC::TcpConnectionListener;
 use UR::Singleton;
-use UR::Test;
 use UR::Util;
 use UR::Value;
 use UR::Value::ARRAY;
