@@ -2,13 +2,15 @@ package UR::BoolExpr;
 use warnings;
 use strict;
 
+use List::MoreUtils qw(uniq);
+use List::Util qw(first);
 use Scalar::Util qw(blessed);
 require UR;
 
 use Carp;
 our @CARP_NOT = ('UR::Context');
 
-our $VERSION = "0.40"; # UR $VERSION;;
+our $VERSION = "0.46"; # UR $VERSION;;
 
 # readable stringification
 use overload ('""' => '__display_name__');
@@ -80,7 +82,7 @@ sub reframe {
 sub __display_name__ {
     my $self = shift;
     my %b = $self->_params_list;
-    my $s = Data::Dumper->new([\%b])->Terse(1)->Indent(0)->Useqq(1)->Dump;
+    my $s = Data::Dumper->new([\%b])->Terse(1)->Indent(0)->Useqq(1)->Sortkeys(1)->Dump;
     $s =~ s/\n/ /gs;
     $s =~ s/^\s*{//;
     $s =~ s/\}\s*$//;
@@ -101,7 +103,7 @@ sub evaluate {
 sub template_and_values {
     my $self = shift;
     my ($template_id, $value_id) = UR::BoolExpr::Type->resolve_ordered_values_from_composite_id($self->id);
-    return (UR::BoolExpr::Template->get($template_id), UR::BoolExpr::Util->value_id_to_values($value_id));
+    return (UR::BoolExpr::Template->get($template_id), UR::BoolExpr::Util::value_id_to_values($value_id));
 }
 
 # Returns true if the rule represents a subset of the things the other
@@ -148,7 +150,7 @@ sub values {
     my $value_id = $self->value_id;
     return unless defined($value_id) and length($value_id);
     my @values;
-    @values = UR::BoolExpr::Util->value_id_to_values($value_id);
+    @values = UR::BoolExpr::Util::value_id_to_values($value_id);
     if (my $hard_refs = $self->{hard_refs}) {
         for my $n (keys %$hard_refs) {
             $values[$n] = $hard_refs->{$n};
@@ -305,32 +307,47 @@ sub flatten_hard_refs {
     for my $key (keys %params) {
         my $value = $params{$key};
         if (ref($value) and Scalar::Util::blessed($value) and $value->isa("UR::Object")) {
-            my ($property_name,$op) = ($key =~ /^(\S+)\s*(.*)/);
-            
-            my $value_class_name = $meta->property($property_name)->data_type;
-            next unless $value_class_name;
-            my $id = $value->id;
-            my $value2 = eval { 
-                $value_class_name->get($id)
+            my ($property_name, $op) = ($key =~ /^(\S+)\s*(.*)/);
+
+            my $pmeta = $meta->property($property_name);
+            my $final_pmeta = $pmeta->final_property_meta();
+
+            my @possible_data_types = uniq grep { $_ } map { $_->data_type } ($pmeta, $final_pmeta);
+            unless (@possible_data_types) {
+                # this might not be possible at runtime
+                croak sprintf 'unable to determine data type for property: %s', $property_name;
+            }
+
+            my $data_type = first { $value->isa($_) } @possible_data_types;
+            unless ($data_type) {
+                croak sprintf 'value type, %s, is incompatible with: %s', $value->class, join(', ', @possible_data_types);
+            }
+
+            my $value2 = do {
+                local $@;
+                eval {
+                    $data_type->get($value->id)
+                };
             };
-            if (not $value2) {
-                next;
+            unless ($value2) {
+                croak sprintf 'unable to retrieve a %s by value ID: %s', $data_type, $value->id;
             }
-            if ($value2 == $value) {
-                # safe to re-represent as .id
-                my $new_key = $property_name . '.id';
-                $new_key .= ' ' . $op if $op;
-                my $new_value = $value->id;
-                delete $params{$key};
-                $params{$new_key} = $new_value;
-                $changes++;
+
+            unless ($value2 eq $value) {
+                croak sprintf 'retrieved duplicate %s with ID, %s,', $data_type, $value->id;
             }
+
+            # safe to re-represent as .id
+            my $new_key = $property_name . '.id';
+            $new_key .= ' ' . $op if $op;
+            delete $params{$key};
+            $params{$new_key} = $value->id;
+            $changes++;
         }
     }
     if ($changes) {
-        return $self->resolve($subject_class_name, %params);
-    }
-    else {
+        return $self->resolve_normalized($subject_class_name, %params);
+    } else {
         return $self;
     }
 }
@@ -351,7 +368,7 @@ sub resolve_normalized {
 
 sub resolve_for_template_id_and_values {
     my ($class,$template_id, @values)  = @_;
-    my $value_id = UR::BoolExpr::Util->values_to_value_id(@values);
+    my $value_id = UR::BoolExpr::Util::values_to_value_id(@values);
     my $rule_id = $class->__meta__->resolve_composite_id_from_ordered_values($template_id,$value_id);
     $class->get($rule_id);
 }
@@ -445,7 +462,7 @@ sub resolve {
             Carp::croak("Can't resolve BoolExpr: undef is an invalid key/property name.  Args were: ".join(', ',@in_params));
         }
 
-        if (substr($key,0,1) eq '-') {
+        if (UR::BoolExpr::Util::is_meta_param($key)) {
             # these are keys whose values live in the rule template
             push @keys, $key;
             push @constant_values, $value;
@@ -477,7 +494,9 @@ sub resolve {
                 if ( $class->_value_is_old_style_operator_and_value($value)) {
                     # the key => { operator => $o, value => $v } syntax
                     # cannot be used with a value type of HASH
-                    $operator = lc($value->{operator});
+                    $operator = defined($value->{operator})
+                                ? lc($value->{operator})
+                                : '';
                     if (exists $value->{escape}) {
                         $operator .= "-" . $value->{escape}
                     }
@@ -526,9 +545,14 @@ sub resolve {
     # this next section uses class metadata
     # it should be moved into the normalization layer
 
-    my $subject_class_meta = eval { $subject_class->__meta__ };
-    if ($@) {
-        Carp::croak("Can't get class metadata for $subject_class.  Is it a valid class name?\nErrors were: $@");
+    my $subject_class_meta;
+    my $exception = do {
+        local $@;
+        $subject_class_meta = eval { $subject_class->__meta__ };
+        $@;
+    };
+    if ($exception) {
+        Carp::croak("Can't get class metadata for $subject_class.  Is it a valid class name?\nErrors were: $exception");
     }
     unless ($subject_class_meta) {
         Carp::croak("No class metadata for $subject_class?!");
@@ -540,11 +564,11 @@ sub resolve {
 
     my($kn, $vn, $cn, $complex_values) = (0,0,0,0);
     my ($op,@extra,@xadd_keys,@xadd_values,@xremove_keys,@xremove_values,@extra_key_pos,@extra_value_pos,
-        @swap_key_pos,@swap_key_value,%in_clause_values_are_strings);
+        @swap_key_pos,@swap_key_value);
 
     for my $value (@values) {
         $key = $keys[$kn++];
-        if (substr($key,0,1) eq '-') {
+        if (UR::BoolExpr::Util::is_meta_param($key)) {
             $cn++;
             redo;
         }
@@ -569,7 +593,9 @@ sub resolve {
 
         # account for the case where this parameter does
         # not match an actual property
-        if (!exists $subject_class_props->{$property_name} and index($property_name,'.') == -1) {
+        my $base_property_name = $property_name;
+        $base_property_name =~ s/[.-].+//;
+        if (!exists $subject_class_props->{$base_property_name}) {
             if (substr($property_name,0,1) eq '_') {
                 warn "ignoring $property_name in $subject_class bx construction!"
             }
@@ -597,8 +623,10 @@ sub resolve {
                     $is_many = $property_meta->is_many;
                 }
                 else {
-                    $data_type = $subject_class_meta->{has}{$property_name}{data_type};
-                    $is_many = $subject_class_meta->{has}{$property_name}{is_many};
+                    if (exists $subject_class_meta->{has}{$property_name}) {
+                        $data_type = $subject_class_meta->{has}{$property_name}{data_type};
+                        $is_many = $subject_class_meta->{has}{$property_name}{is_many};
+                    }
                 }
                 $data_type ||= '';
 
@@ -623,17 +651,15 @@ sub resolve {
                     $value = [ sort $sorter @$value ];
 
                     # Remove duplicates from the list
-                    if ($operator ne 'between' and $operator ne 'not between') {
-                        my $last = $value;
-                        for (my $i = 0; $i < @$value;) {
-                            if ($last eq $value->[$i]) {
-                                splice(@$value, $i, 1);
-                            }
-                            else {
-                                $last = $value->[$i++];
-                             }
-                         }
-                     }
+                    my $last = $value;
+                    for (my $i = 0; $i < @$value;) {
+                        if ($last eq $value->[$i]) {
+                            splice(@$value, $i, 1);
+                        }
+                        else {
+                            $last = $value->[$i++];
+                        }
+                    }
                     # push @swap_key_pos, $vn-1;
                     # push @swap_key_value, $property_name;
                 }
@@ -674,26 +700,15 @@ sub resolve {
                     if (my $id_class_by = $property_meta->id_class_by) {
                         push @xadd_keys, $id_class_by;
                         push @xadd_values, ref($value);
-                        #print ":: @xkeys\n::@xvalues\n\n";
                     }
                     push @xremove_keys, $kn-1;
                     push @xremove_values, $vn-1;
                 }
-                # This is disabled here because it is good for get() but not create()
-                # The flatten_hard_refs() method is run before doing a get() to create the same effect.
-                # elsif ($property_meta->is_delegated and not $property_meta->is_many) {
-                #    print STDERR "adding $property_name.id\n";
-                #    push @xadd_keys, $property_name . '.id' . ' ' . $operator;
-                #    push @xadd_values, $value->id;
-                #    push @xremove_keys, $kn-1;
-                #    push @xremove_values, $vn-1;
-                # }
                 elsif ($property_meta->is_valid_storage_for_value($value)) {
                     push @hard_refs, $vn-1, $value;
                 }
                 elsif ($value->can($property_name)) {
                     # TODO: stop suporting foo_id => $foo, since you can do foo=>$foo, and foo_id=>$foo->id
-                    #$DB::single = 1;
                     # Carp::cluck("using $property_name => \$obj to get $property_name => \$obj->$property_name is deprecated...");
                     $value = $value->$property_name;
                 }
@@ -728,17 +743,21 @@ sub resolve {
     }
 
     if (@xremove_keys) {
-        my @new;
-        my $next_pos_to_remove = $xremove_keys[0];
-        for (my $n = 0; $n < @keys; $n++) {
-            if (defined $next_pos_to_remove and $n == $next_pos_to_remove) {
-                shift @xremove_keys;
-                $next_pos_to_remove = $xremove_keys[0];
+        my $write_key_idx = 0;
+        for (my($read_key_idx, $xremove_key_idx) = (0,0);
+             $read_key_idx < @keys;
+             $read_key_idx++
+        ) {
+            if ($xremove_key_idx < @xremove_keys
+                and
+                $read_key_idx == $xremove_keys[$xremove_key_idx]
+            ) {
+                $xremove_key_idx++;
                 next;
             }
-            push @new, $keys[$n];
+            $keys[$write_key_idx++] = $keys[$read_key_idx];
         }
-        @keys = @new;
+        $#keys = $write_key_idx-1;
     }
 
     if (@xremove_values) {
@@ -760,18 +779,21 @@ sub resolve {
             }
         }
 
-
-        my @new;
-        my $next_pos_to_remove = $xremove_values[0];
-        for (my $n = 0; $n < @values; $n++) {
-            if (defined $next_pos_to_remove and $n == $xremove_values[0]) {
-                shift @xremove_values;
-                $next_pos_to_remove = $xremove_values[0];
+        my $write_value_idx = 0;
+        for(my($read_value_idx, $xremove_value_idx) = (0,0);
+            $read_value_idx < @values;
+            $read_value_idx++
+        ) {
+            if ($xremove_value_idx < @xremove_values
+                and
+                $read_value_idx == $xremove_values[$xremove_value_idx]
+            ) {
+                $xremove_value_idx++;
                 next;
             }
-            push @new, $values[$n];
+            $values[$write_value_idx++] = $values[$read_value_idx];
         }
-        @values = @new;
+        $#values = $write_value_idx-1;
     }
 
     my $template;
@@ -791,7 +813,7 @@ sub resolve {
             );
     }
 
-    my $value_id = ($complex_values ? UR::BoolExpr::Util->values_to_value_id(@values) : UR::BoolExpr::Util->values_to_value_id_simple(@values) );
+    my $value_id = UR::BoolExpr::Util::values_to_value_id(@values);
 
     my $rule_id = join($UR::BoolExpr::Util::id_sep,$template->{id},$value_id);
 
@@ -799,14 +821,13 @@ sub resolve {
 
     $rule->{template} = $template;
     $rule->{values} = \@values;
-    $rule->{_in_clause_values_are_strings} = \%in_clause_values_are_strings if (keys %in_clause_values_are_strings);
 
     $vn = 0;
     $cn = 0;
     my @list;
     for my $key (@keys) {
         push @list, $key;
-        if (substr($key,0,1) eq '-') {
+        if (UR::BoolExpr::Util::is_meta_param($key)) {
             push @list, $constant_values[$cn++];
         }
         else {
@@ -846,7 +867,7 @@ sub _params_list {
             my $cn = 0;
             for my $key (@$k) {
                 push @list, $key;
-                if (substr($key,0,1) eq '-') {
+                if (UR::BoolExpr::Util::is_meta_param($key)) {
                     push @list, $c->[$cn++];
                 }
                 else {
@@ -919,9 +940,13 @@ sub resolve_for_string {
     my ($class, $subject_class_name, $filter_string, $usage_hints_string, $order_string, $page_string) = @_;
 
     unless ($LOADED_BXPARSE) {
-        eval { require UR::BoolExpr::BxParser };
-        if ($@) {
-            Carp::croak("resolve_for_string() can't load UR::BoolExpr::BxParser: $@");
+        my $exception = do {
+            local $@;
+            eval { require UR::BoolExpr::BxParser };
+            $@;
+        };
+        if ($exception) {
+            Carp::croak("resolve_for_string() can't load UR::BoolExpr::BxParser: $exception");
         }
         $LOADED_BXPARSE=1;
     }
@@ -937,7 +962,12 @@ sub resolve_for_string {
     push @$tree, '-order_by', [split(',',$order_string) ] if ($order_string);
     push @$tree, '-page',     [split(',',$page_string) ] if ($page_string);
 
-    my $bx = UR::BoolExpr->resolve($subject_class_name, @$tree);
+    my ($bx, @extra);
+    if(wantarray) {
+        ($bx, @extra) = UR::BoolExpr->resolve($subject_class_name, @$tree);
+    } else {
+        $bx = UR::BoolExpr->resolve($subject_class_name, @$tree);
+    }
     unless ($bx) {
         Carp::croak("Can't create BoolExpr on $subject_class_name from params generated from string "
                     . $filter_string . " which parsed as:\n"
@@ -946,46 +976,11 @@ sub resolve_for_string {
     if ($$remaining_strref) {
         Carp::croak("Trailing input after the parsable end of the filter string: '". $$remaining_strref."'");
     }
-    return $bx;
-}
-
-# TODO: these methods need a better home, since they are a cmdline/UI standard
-sub _old_filter_regex_for_string {
-    return '^\s*([\w\.\-]+)\s*(\@|\=|!=|=|\>|\<|~|!~|!\:|\:|\blike\b|\bbetween\b|\bin\b)\s*[\'"]?([^\'"]*)[\'"]?\s*$';
-}
-
-# TODO: these methods need a better home, since they are a cmdline/UI standard
-sub _old_resolve_for_string {
-    my ($self, $subject_class_name, $filter_string, $usage_hints_string, $order_string, $page_string) = @_;
-
-    my ($property, $op, $value);
-
-    no warnings;
-
-    my $filter_regex = $self->_old_filter_regex_for_string();
-    my @filters = map {
-        unless (($property, $op, $value) = ($_ =~ /$filter_regex/)) {
-            Carp::croak "Unable to process filter $_\n";
-        }
-        if ($op eq '~') {
-            $op = "like";
-            # If the user asked for 'like', but didn't put in a wildcard, then put wildcards
-            # on each side of the value
-            $value = '%'.$value.'%' if (length($value) and $value !~ m/\%|_/);
-        } elsif ($op eq '!~') {
-            $op = 'not like';
-            $value = '%'.$value.'%' if (length($value) and $value !~ m/\%|_/);
-        }
-
-        [$property, $op, $value]
-    } split(/,/, $filter_string);
-
-    my @hints = split(",",$usage_hints_string);
-    my @order = split(",",$order_string);
-    my @page  = split(",",$page_string);
-
-    use warnings;
-    return __PACKAGE__->_resolve_from_filter_array($subject_class_name, \@filters, \@hints, \@order, \@page);
+    if(wantarray) {
+        return ($bx, @extra);
+    } else {
+        return $bx;
+    }
 }
 
 sub _resolve_from_filter_array {
@@ -1122,8 +1117,8 @@ sub _resolve_from_subject_class_name_keys_and_values {
     my @keys            = @{ $params{keys} || [] };
     die "unexpected params: " . Data::Dumper::Dumper(\%params) if %params;
 
-    my $value_id = UR::BoolExpr::Util->values_to_value_id(@values);
-    my $constant_value_id = UR::BoolExpr::Util->values_to_value_id(@constant_values);
+    my $value_id = UR::BoolExpr::Util::values_to_value_id(@values);
+    my $constant_value_id = UR::BoolExpr::Util::values_to_value_id(@constant_values);
 
     my $template_id = $subject_class_name . '/And/' . join(",",@keys) . "/" . $constant_value_id;
     my $rule_id = join($UR::BoolExpr::Util::id_sep,$template_id,$value_id);
@@ -1188,7 +1183,7 @@ UR::BoolExpr - a "where clause" for objects
     my $bx3 = $bx->flatten();
     # any indirection in the params takes the form a.b.c at the lowest level
     # also 'payroll_category' might become 'pay_history.category', and 'pay_history.is_current' => 1 is added to the list
-    # if this paramter has that as a custom filter
+    # if this parameter has that as a custom filter
 
 
 =head1 DESCRIPTION
@@ -1196,11 +1191,11 @@ UR::BoolExpr - a "where clause" for objects
 A UR::BoolExpr object captures a set of match criteria for some class of object.
 
 Calls to get(), create(), and define_set() all use this internally to objectify
-their paramters.  If given a boolean expression object directly they will use it.
+their parameters.  If given a boolean expression object directly they will use it.
 Otherwise they will construct one from the parameters given.
 
 They have a 1:1 correspondence within the WHERE clause in an SQL statement where
-RDBMS persistance is used.  They also imply the FROM clause in these cases,
+RDBMS persistence is used.  They also imply the FROM clause in these cases,
 since the query properties control which joins must be included to return
 the matching object set.
 
@@ -1260,7 +1255,7 @@ them.  For example:
     name_property in [Bob,Fred,Joe]
 
 Simple expressions may be joined together with the words "and" and "or" to form a more
-complicated expression.  "and" has higher precedence than "or", and parentheses can 
+complicated expression.  "and" has higher precedence than "or", and parentheses can
 surround sub-expressions to indicate the requested precedence.  For example:
     ((prop1 = foo or prop2 = 1) and (prop2 > 10 or prop3 like 'Yo%')) or prop4 in [1,2,3]
 
@@ -1340,17 +1335,17 @@ Return the value for the given property
   my $operator = $bx->operator_for('property_name');
 
 Return a string for the operator of the given property.  A value of '' (the
-empty string) means equality ("=").  Other possible values inclue '<', '>',
+empty string) means equality ("=").  Other possible values include '<', '>',
 '<=', '>=', 'between', 'true', 'false', 'in', 'not <', 'not >', etc.
 
 =item normalize
 
     $bx2 = $bx->normalize;
 
-A boolen expression can be changed in incidental ways and still be equivalent.
+A boolean expression can be changed in incidental ways and still be equivalent.
 This method converts the expression into a normalized form so that it can be
 compared to other normalized expressions without incidental differences
-affecting the comparision.
+affecting the comparison.
 
 =item flatten
 
