@@ -6,7 +6,7 @@ use warnings;
 use Scalar::Util;
 use UR::Context;
 
-our $VERSION = "0.40"; # UR $VERSION;
+our $VERSION = "0.46"; # UR $VERSION;
 
 # A helper package for UR::Context to keep track of the subrefs used
 # to create objects from database data
@@ -47,7 +47,6 @@ sub _create {
 
 sub create_for_loading_template {
     my($fab_class, $context, $loading_template, $query_plan, $rule, $rule_template, $values, $dsx) = @_;
-
     my @values = @$values;
 
     my $class_name                                  = $loading_template->{final_class_name};
@@ -151,6 +150,9 @@ sub create_for_loading_template {
         @all_rule_property_names = $rule_template_without_in_clause->_property_names;
         foreach my $property ( @rule_properties_with_in_clauses ) {
             my $values_for_in_clause = $rule_without_recursion_desc->value_for($property);
+            unless ($values_for_in_clause) {
+                Carp::confess("rule has no value for property $property: $rule_without_recursion_desc");
+            }
             if (@$values_for_in_clause > 100) {
                 $do_record_in_all_params_loaded = 0;
                 next;
@@ -709,7 +711,6 @@ sub _lapl_data_for_delegation_data {
 
     my %tmpl_and_rules;
 
-#$DB::single=1;
     foreach my $delegation_data ( @$delegation_data_list ) {
         my $value_sources = $delegation_data->[0];
         my $rule_tmpl  = $delegation_data->[1];
@@ -1120,7 +1121,7 @@ that was using this fabricator has gone out of scope.
   $fab->apply_all_params_loaded();
 
 As the fabricator constructs objects, it buffers changes to all_params_loaded
-(the Context's query cache) to maintain consistancy if multiple iterators are
+(the Context's query cache) to maintain consistency if multiple iterators are
 working concurrently.  At the appripriate time, call apply_all_params_loaded()
 to take those changes and apply them to the current Context's all_params_loaded.
 
