@@ -16,7 +16,7 @@ package UR::DataSource::File;
 use UR;
 use strict;
 use warnings;
-our $VERSION = "0.40"; # UR $VERSION;
+our $VERSION = "0.46"; # UR $VERSION;
 
 use Fcntl qw(:DEFAULT :flock);
 use Errno qw(EINTR EAGAIN EOPNOTSUPP);
@@ -54,56 +54,60 @@ class UR::DataSource::File {
 
 sub can_savepoint { 0;}  # Doesn't support savepoints
  
-sub get_default_handle {
+sub create_default_handle {
     my $self = shift;
 
-    unless ($self->{'_fh'}) {
-        if ($ENV{'UR_DBI_MONITOR_SQL'}) {
-            my $time = time();
-            UR::DBI->sql_fh->printf("\nFILE OPEN AT %d [%s]\n",$time, scalar(localtime($time)));
-        }
-
-        my $filename = $self->server;
-        unless (-e $filename) {
-            # file doesn't exist
-            $filename = '/dev/null';
-        }
-
-        my $handle_class = $self->handle_class;
-        my $fh = $handle_class->new($filename);
-        unless($fh) {
-            $self->error_message("Can't open ".$self->server." for reading: $!");
-            return;
-        }
-
-        if ($ENV{'UR_DBI_MONITOR_SQL'}) {
-            UR::DBI->sql_fh->printf("FILE: opened %s fileno %d\n\n",$self->server, $fh->fileno);
-        }
-
-        $self->{'_fh'} = $fh;
-        $self->is_connected(1);
+    if ($ENV{'UR_DBI_MONITOR_SQL'}) {
+        my $time = time();
+        UR::DBI->sql_fh->printf("\nFILE OPEN AT %d [%s]\n",$time, scalar(localtime($time)));
     }
-    return $self->{'_fh'};
+
+    my $filename = $self->server;
+    unless (-e $filename) {
+        # file doesn't exist
+        $filename = '/dev/null';
+    }
+
+    my $handle_class = $self->handle_class;
+    my $fh = $handle_class->new($filename);
+    unless($fh) {
+        $self->error_message("Can't open ".$self->server." for reading: $!");
+        return;
+    }
+
+    if ($ENV{'UR_DBI_MONITOR_SQL'}) {
+        UR::DBI->sql_fh->printf("FILE: opened %s fileno %d\n\n",$self->server, $fh->fileno);
+    }
+
+    $self->is_connected(1);
+    return $fh;
 }
 
-sub disconnect_default_handle {
+sub disconnect {
     my $self = shift;
 
-    if (my $fh = $self->{'_fh'}) {
+    if ($self->has_default_handle) {
+        my $fh = $self->get_default_handle;
         flock($fh,LOCK_UN);
         $fh->close();
-        $self->{'_fh'} = undef;
+        $self->__invalidate_get_default_handle__;
         $self->is_connected(0);
     }
 }
 
+sub _file_position {
+    my $self = shift;
+    my $fh = $self->get_default_handle;
+    return $fh ? $fh->tell() : undef;
+}
+
 sub prepare_for_fork {
     my $self = shift;
 
     # make sure this is clear before we fork
     $self->{'_fh_position'} = undef;
-    if (defined $self->{'_fh'}) {
-        $self->{'_fh_position'} = $self->{'_fh'}->tell();
+    if ($self->has_default_handle) {
+        $self->{'_fh_position'} = $self->_file_position();
         UR::DBI->sql_fh->printf("FILE: preparing to fork; closing file %s and noting position at %s\n",$self->server, $self->{'_fh_position'}) if $ENV{'UR_DBI_MONITOR_SQL'};
     }
     $self->disconnect_default_handle;
@@ -554,7 +558,7 @@ sub _comparator_for_operator_and_property {
     }
 }
         
-
+my $iterator_serial = 0;
 sub create_iterator_closure_for_rule {
     my($self,$rule) = @_;
 
@@ -694,7 +698,7 @@ sub create_iterator_closure_for_rule {
         UR::DBI->sql_fh->printf("\nFILE: %s\nFILTERS %s\n\n", $self->server, $filter_list);
     }
 
-    $self->{'_last_read_fingerprint'} ||= '';
+    $self->{'_last_read_serial'} ||= '';
 
     my $record_separator = $self->record_separator;
     my $cache_slot = $self->_allocate_offset_cache_slot();
@@ -705,7 +709,7 @@ sub create_iterator_closure_for_rule {
     my $lines_matched = 0;
 
     my $fh;  # File handle we'll be reading from
-    my $read_fingerprint;   # The stringified version of $iterator (to avoid circular references), filled in below
+    my $this_iterator_serial = $iterator_serial++;
     my $iterator = sub {
 
         unless (ref($fh)) {
@@ -721,7 +725,7 @@ sub create_iterator_closure_for_rule {
             $monitor_printed_first_fetch = 1;
         }
 
-        if ($self->{'_last_read_fingerprint'} ne $read_fingerprint) {
+        if ($self->{'_last_read_serial'} ne $this_iterator_serial) {
             UR::DBI->sql_fh->printf("FILE: Resetting file position to $file_pos\n") if $ENV{'UR_DBI_MONITOR_SQL'};
             # The last read was from a different request, reset the position
             $fh->seek($file_pos,0);
@@ -731,9 +735,9 @@ sub create_iterator_closure_for_rule {
                     scalar(<$fh>);
                 }
             }
-            $file_pos = $fh->tell();
+            $file_pos = $self->_file_position();
 
-            $self->{'_last_read_fingerprint'} = $read_fingerprint;
+            $self->{'_last_read_serial'} = $this_iterator_serial;
         }
 
         local $/;   # Make sure some wise guy hasn't changed this out from under us
@@ -780,7 +784,7 @@ sub create_iterator_closure_for_rule {
             $next_candidate_row = [ split($split_regex, $line, $csv_column_count) ];
             $#{$a} = $csv_column_count-1;
 
-            $file_pos = $fh->tell();
+            $file_pos = $self->_file_position();
             my $file_pos_before_read = $file_pos - $last_read_size;
 
             # Every so many lines read, leave a breadcrumb about what we've seen
@@ -844,7 +848,6 @@ sub create_iterator_closure_for_rule {
             return $next_candidate_row;
         }
     }; # end sub $iterator
-    $read_fingerprint = $iterator . '';
 
     Sub::Name::subname('UR::DataSource::File::__datasource_iterator(closure)__', $iterator);
 
@@ -875,15 +878,15 @@ sub UR::DataSource::File::Tracker::DESTROY {
     $ds->_open_query_count(--$count);
 
     return unless ($ds->quick_disconnect);
-    if ($count == 0) {
+    if ($count == 0 && $ds->has_default_handle) {
 	# All open queries have supposedly been fulfilled.  Close the
 	# file handle and undef it so get_default_handle() will re-open if necessary
-        my $fh = $ds->{'_fh'};
+        my $fh = $ds->get_default_handle;
 
         UR::DBI->sql_fh->printf("FILE: CLOSING fileno ".fileno($fh)."\n") if ($ENV{'UR_DBI_MONITOR_SQL'});
         #flock($fh,LOCK_UN);
-	$fh->close();
-	$ds->{'_fh'} = undef;
+        $fh->close();
+        $ds->__invalidate_get_default_handle__;
     }
 }
 
@@ -1203,7 +1206,7 @@ sub _sync_database {
     # Because of the rename/copy process during syncing, the previously opened filehandle may
     # not be valid anymore.  get_default_handle will reopen the file next time it's needed
     $self->_invalidate_cache();
-    $self->{_fh} = undef; 
+    $self->__invalidate_get_default_handle__;
 
     if ($ENV{'UR_DBI_MONITOR_SQL'}) {
         UR::DBI->sql_fh->printf("FILE: TOTAL COMMIT TIME: %.4f s\n", Time::HiRes::time() - $monitor_start_time);
