@@ -4,14 +4,14 @@ use warnings;
 package UR::DataSource::RDBMS::PkConstraintColumn;
 
 use UR;
-our $VERSION = "0.40"; # UR $VERSION;
+our $VERSION = "0.46"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::DataSource::RDBMS::PkConstraintColumn',
     is => ['UR::DataSource::RDBMS::Entity'],
     dsmap => 'dd_pk_constraint_column',
     er_role => '',
-    id_properties => [qw/data_source owner table_name column_name rank/],
+    id_properties => [qw/data_source table_name column_name rank/],
     properties => [
         column_name                      => { type => 'varchar', len => undef, sql => 'column_name' },
         data_source                      => { type => 'varchar', len => undef, sql => 'data_source' },
