@@ -4,13 +4,13 @@ use warnings;
 package UR::DataSource::RDBMS::Table;
 
 use UR;
-our $VERSION = "0.40"; # UR $VERSION;
+our $VERSION = "0.46"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::DataSource::RDBMS::Table',
     is => ['UR::DataSource::RDBMS::Entity'],
     dsmap => 'dd_table',
-    id_properties => [qw/data_source owner table_name/],
+    id_properties => [qw/data_source table_name/],
     properties => [
         data_source                      => { type => 'varchar', len => undef, sql => 'data_source' },
         data_source_obj                  => { type => 'UR::DataSource', id_by => 'data_source'}, 
@@ -92,7 +92,7 @@ sub fk_constraints {
     my $self = shift;
 
     my $fk_class = $self->_fk_constraint_class;
-    my @fks = $fk_class->get(data_source => $self->data_source, table_name => $self->table_name, owner => $self->owner);
+    my @fks = $fk_class->get(data_source => $self->data_source, table_name => $self->table_name);
     return @fks;
 }
 
@@ -105,7 +105,7 @@ sub ref_fk_constraints {
     my $self = shift;
 
     my $fk_class = $self->_fk_constraint_class;
-    my @fks = $fk_class->get(data_source => $self->data_source, r_table_name => $self->table_name, r_owner => $self->owner);
+    my @fks = $fk_class->get(data_source => $self->data_source, r_table_name => $self->table_name);
     return @fks;
 }
 
