@@ -37,15 +37,13 @@ a formal UR class.
 
 =head1 METHODS
 
-=over 4
-
 =cut
 
 # set up package
 require 5.006_000;
 use warnings;
 use strict;
-our $VERSION = "0.40"; # UR $VERSION;;
+our $VERSION = "0.46"; # UR $VERSION;;
 
 # set up module
 use Carp;
@@ -54,6 +52,8 @@ use UR::Util;
 
 =pod
 
+=over
+
 =item C<class>
 
   $class = $obj->class;
@@ -243,14 +243,14 @@ sub context_return {
 
 =head1 C<AUTOLOAD>
 
-This package impliments AUTOLOAD so that derived classes can use
+This package implements AUTOLOAD so that derived classes can use
 AUTOSUB instead of AUTOLOAD.
 
 When a class or object has a method called which is not found in the
 final class or any derived classes, perl checks up the tree for
-AUTOLOAD.  We impliment AUTOLOAD at the top of the tree, and then
+AUTOLOAD.  We implement AUTOLOAD at the top of the tree, and then
 check each class in the tree in order for an AUTOSUB method.  Where a
-class implements AUTOSUB, it will recieve a function name as its first
+class implements AUTOSUB, it will receive a function name as its first
 parameter, and it is expected to return either a subroutine reference,
 or undef.  If undef is returned then the inheritance tree search will
 continue.  If a subroutine reference is returned it will be executed
@@ -264,7 +264,7 @@ Why not use AUTOLOAD directly in place of AUTOSUB?
 On an object with a complex inheritance tree, AUTOLOAD is only found
 once, after which, there is no way to indicate that the given AUTOLOAD
 has failed and that the inheritance tree trek should continue for
-other AUTOLOADS which might impliment the given method.
+other AUTOLOADS which might implement the given method.
 
 Example:
 
@@ -396,7 +396,7 @@ With no arguments, this method returns all the types of messages that
 this class handles.  With arguments, it adds a new type to the
 list.
 
-Standard message types are error, status, warning, debug and usage.
+Standard message types are fatal, error, status, warning, debug and usage.
 
 Note that the addition of new types is not fully supported/implemented
 yet.
@@ -406,7 +406,7 @@ yet.
 =cut
 
 my $create_subs_for_message_type;  # filled in lower down
-my @message_types = qw(error status warning debug usage);
+my @message_types = qw(error status warning debug usage fatal);
 sub message_types
 {
     my $self = shift;
@@ -430,6 +430,7 @@ my %default_messaging_settings;
 $default_messaging_settings{dump_error_messages} = 1;
 $default_messaging_settings{dump_warning_messages} = 1;
 $default_messaging_settings{dump_status_messages} = 1;
+$default_messaging_settings{dump_fatal_messages} = 1;
 
 #
 # Implement error_mesage/warning_message/status_message in a way
@@ -464,14 +465,21 @@ For example, for the "error" message type, these methods are created:
 =item error_message
 
     $obj->error_message("Something went wrong...");
+    $obj->error_message($format, @list);
     $msg = $obj->error_message();
 
-When called with one argument, it sends an error message to the object.  The 
-error_message_callback will be run, if one is registered, and the message will
-be printed to the terminal.  When called with no arguments, the last message
-sent will be returned.  If the message is C<undef> then no message is printed
-or queued, and the next time error_message is run as an accessor, it will
-return undef.
+When called with one or more arguments, it sends an error message to the
+object.  The error_message_callback will be run, if one is registered, and the
+message will be printed to the terminal.  When given a single argument, it will
+be passed through unmodified.  When given multiple arguments, error_message will
+assume the first is a format string and the remainder are parameters to
+sprintf.  When called with no arguments, the last message sent will be
+returned.  If the message is C<undef> then no message is printed or queued, and
+the next time error_message is run as an accessor, it will return
+undef.
+
+Note that C<fatal_message()> will throw an exception at the point it appears
+in the program.  This exception, like others, is trappable bi C<eval>.
 
 =item dump_error_messages
 
@@ -482,6 +490,10 @@ Get or set the flag which controls whether messages sent via C<error_message()>
 is printed to the terminal.  This flag defaults to true for warning and error
 messages, and false for others.
 
+Note that C<fatal_message()> messages and exceptions do not honor the value of
+C<dump_fatal_messages()>, and always print their message and throw their
+exception unless trapped with an C<eval>.
+
 =item queue_error_messages
 
     $obj->queue_error_messages(0);
@@ -513,6 +525,10 @@ with undef as the argument.
 If the queue_error_messages flag is on, then this method returns the entire list
 of queued messages.
 
+When called as an instance method, it returns the errors queued only on that
+object.  When called as a class method, it returns the errors queued on that
+class, all it's subclasses, and all instances of that class or subclasses.
+
 =item error_messages_arrayref
 
     $listref = $obj->error_messages_arrayref();
@@ -662,8 +678,17 @@ $create_subs_for_message_type = sub {
     my $array_subname = "${type}_messages";
     my $array_subref = Sub::Name::subname "${class}::${array_subname}" => sub {
         my $self = shift;
-        my $a = $get_setting->($self, $messages_arrayref);
-        return $a ? @$a : ();
+        my @search = ref($self)
+                        ? $self
+                        : ( $self, $self->__meta__->subclasses_loaded, $self->is_loaded() );
+        my %seen;
+        my @all_messages;
+        foreach my $thing ( @search ) {
+            next if $seen{$thing}++;
+            my $a = $get_setting->($thing, $messages_arrayref);
+            push @all_messages, $a ? @$a : ();
+        }
+        return @all_messages;
     };
     Sub::Install::install_sub({
         code => $array_subref,
@@ -695,13 +720,31 @@ $create_subs_for_message_type = sub {
     my $message_line        = "${type}_line";
     my $message_subroutine  = "${type}_subroutine";
 
+    my $messaging_action = $type eq 'fatal'
+                            ? sub { Carp::croak($message_text_prefix . $_[1]) }
+                            : sub {
+                                my($self, $msg) = @_;
+                                if (my $fh = $self->$should_dump_messages()) {
+                                    $fh = $$default_fh unless (ref $fh);
+
+                                    $fh->print($message_text_prefix . $msg . "\n");
+                                }
+                            };
+
     my $logger_subname = "${type}_message";
     my $logger_subref = Sub::Name::subname "${class}::${logger_subname}" => sub {
         my $self = shift;
 
-        foreach ( @_ ) {
-            my $msg = $_;
-            chomp($msg) if defined;
+        if (@_) {
+            my $msg = shift;
+
+            # if given multiple arguments, assume it's a format string
+            if(@_) {
+                $msg = _carp_sprintf($msg, @_);
+            }
+
+            defined($msg) && chomp($msg);
+
             # old-style callback registered with error_messages_callback
             if (my $code = $self->$check_callback()) {
                 if (ref $code) {
@@ -720,24 +763,21 @@ $create_subs_for_message_type = sub {
             $save_setting->($self, $logger_subname, $msg);
             # If the callback set $msg to undef with "$_[1] = undef", then they didn't want the message
             # processed further
-            next unless defined($msg);
+            if (defined $msg) {
+                if ($self->$should_queue_messages()) {
+                    my $a = $self->$messages_arrayref();
+                    push @$a, $msg;
+                }
 
-            if (my $fh = $self->$should_dump_messages()) {
-                $fh = $$default_fh unless (ref $fh);
+                my ($package, $file, $line, $subroutine) = caller;
+                $self->$message_package($package);
+                $self->$message_file($file);
+                $self->$message_line($line);
+                $self->$message_subroutine($subroutine);
+
+                $self->$messaging_action($msg);
 
-                $fh->print($message_text_prefix . $msg . "\n");
             }
-
-            if ($self->$should_queue_messages()) {
-                my $a = $self->$messages_arrayref();
-                push @$a, $msg;
-            }
-
-            my ($package, $file, $line, $subroutine) = caller;
-            $self->$message_package($package);
-            $self->$message_file($file);
-            $self->$message_line($line);
-            $self->$message_subroutine($subroutine);
         }
 
         return $get_setting->($self, $logger_subname);
@@ -747,8 +787,35 @@ $create_subs_for_message_type = sub {
         into => $class,
         as => $logger_subname,
     });
+
+    # "Register" the message type as a valid signal.
+    $UR::Object::Type::STANDARD_VALID_SIGNALS{$logger_subname} = 1;
 };
 
+sub _carp_sprintf {
+    my $format = shift;
+    my @list = @_;
+
+    # warnings weren't very helpful because they wouldn't tell you who passed
+    # in the "bad" format string
+    my $formatted_string;
+    my $warn_msg;
+    {
+        local $SIG{__WARN__} = sub {
+            my $msg = $_[0];
+            my ($filename, $line) = (caller)[1, 2];
+            my $short_msg = ($msg =~ /(.*) at $filename line $line./)[0];
+            $warn_msg = ($short_msg || $msg);
+        };
+        $formatted_string = sprintf($format, @list);
+    }
+    if ($warn_msg) {
+        Carp::carp($warn_msg);
+    }
+
+    return $formatted_string;
+}
+
 
 # at init time, make messaging subs for the initial message types
 $create_subs_for_message_type->(__PACKAGE__, $_) foreach @message_types;
@@ -778,8 +845,6 @@ __END__
 
 =pod
 
-=back
-
 =head1 SEE ALSO
 
 UR(3)
