@@ -5,7 +5,8 @@ use warnings;
 use File::Find;
 
 require UR;
-our $VERSION = "0.40"; # UR $VERSION;
+use UR::AttributeHandlers;
+our $VERSION = "0.46"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::Namespace',
@@ -33,6 +34,18 @@ UR::Object::Type->define(
     doc => 'The class for a singleton module representing a UR namespace.',
 );
 
+sub import {
+    my $class = shift;
+    return if ($class eq 'UR' or $class eq __PACKAGE__);
+
+    my $calling_package;
+    for(my $i = 0; ($calling_package) = caller($i); $i++) {
+        last unless (substr($calling_package, 0, 4) eq 'UR::');
+    }
+    return unless $calling_package;
+    UR::AttributeHandlers::import_support_functions_to_package($calling_package);
+}
+
 sub get_member_class {
     my $self = shift;
     return UR::Object::Type->get(@_);
@@ -219,7 +232,7 @@ the MyApp tree.
 
 A UR namespace is the top-level object that represents your data's class
 structure in the most general way.  After use-ing a namespace module, the
-program gets access to the module autoloader, which will automaticaly use
+program gets access to the module autoloader, which will automatically use
 modules on your behalf if you attempt to interact with their packages in
 a UR-y way, such as calling get().
 
