@@ -5,10 +5,11 @@ use strict;
 
 require UR;
 
-use Scalar::Util;
+use Scalar::Util qw(looks_like_number refaddr isweak);
+use List::MoreUtils qw(any);
 
 our @ISA = ('UR::ModuleBase');
-our $VERSION = "0.40"; # UR $VERSION;;
+our $VERSION = "0.46"; # UR $VERSION;;
 
 # Base object API
 
@@ -28,12 +29,39 @@ sub delete {
     $UR::Context::current->delete_entity(@_);
 }
 
+sub copy {
+    my $self = shift;
+    my %override = @_;
+
+    my $meta = $self->__meta__;
+    my @copyable_properties =
+        grep { !$_->is_delegated && !$_->is_id }
+        $meta->properties;
+
+    my %params;
+    for my $p (@copyable_properties) {
+        my $name = $p->property_name;
+        if ($p->is_many) {
+            if (my @value = $self->$name) {
+                $params{$name} = \@value;
+            }
+        }
+        else {
+            if (defined(my $value = $self->$name)) {
+                $params{$name} = $value;
+            }
+        }
+    }
+
+    return $self->class->create(%params, %override);
+}
+
 
 # Meta API
 
 sub __context__ {
     # In UR, a "context" handles inter-object references so they can cross
-    # process boundaries, and interact with persistance systems automatically.
+    # process boundaries, and interact with persistence systems automatically.
 
     # For efficiency, all context switches update a package-level value.
 
@@ -57,7 +85,24 @@ sub __meta__  {
 # things like mapping operations easier and calculate_from metadata able
 # to include the object as function args to calculated properties
 sub __self__ {
-    return $_[0];
+    return $_[0] if @_ == 1;
+    my $self = shift;
+    my $bx = $self->class->define_boolexpr(@_);
+    if ($bx->evaluate($self)) {
+        return $self;
+    }
+    else {
+        return;
+    }
+}
+
+sub does {
+    my($self, $role_name) = @_;
+
+    my @roles = map { @{ $_->roles } }
+                $self->__meta__->all_class_metas();
+
+    any { $role_name eq $_->role_name } @roles;
 }
 
 
@@ -159,6 +204,11 @@ sub __errors__ {
 
         my $value = $values[0];
 
+        # account for minus sign in dummy ID
+        if ($ENV{UR_USE_DUMMY_AUTOGENERATED_IDS} and $property_metadata->is_id and defined($value) and index($value, '-') == 0 and defined $data_length) {
+            $data_length++;
+        }
+
         if (! ($is_optional or defined($value))) {
             push @tags, UR::Object::Tag->create(
                             type => 'invalid',
@@ -174,51 +224,18 @@ sub __errors__ {
         # Check data type
         # TODO: delegate to the data type module for this
         $generic_data_type = '' unless (defined $generic_data_type);
-        if ($generic_data_type eq 'Float') {
-            $value =~ s/\s//g;
-            $value = $value + 0;
 
-            my $length =0;
-
-            if($value =~ /^(\+|\-)?([0-9]+)(\.([0-9]*))?[eE](\+|\-)?(\d+)$/){ #-- scientific notation
-                $length = length($2)-1 + $6 + (!$5 || $5 eq '+' ? 1 : 0);
-            }
-            elsif($value =~ /^(\+|\-)?([0-9]*)(\.([0-9]*))?$/) {
-                # If the data type is specified as a Float, but really contains an int, then
-                # $4 is undef causing a warning about "uninitialized value in concatenation",
-                # but otherwise works OK
-                no warnings 'uninitialized';
-                $length = length($2.$4);
-                --$length if $2 == 0 && $4;
+        if ($generic_data_type eq 'Float' || $generic_data_type eq 'Integer') {
+            if (looks_like_number($value)) {
+                $value = $value + 0;
             }
             else{
                 push @tags, UR::Object::Tag->create (
                     type => 'invalid',
                     properties => [$property_name],
-                    desc => 'Invalid decimal value.'
+                    desc => "Invalid $generic_data_type value."
                 );
             }
-            # Cleanup for size check below.
-            $value = '.' x $length;
-        }
-        elsif ($generic_data_type eq 'Integer') {
-            $value =~ s/\s//g;
-            if ($value =~ /\D/) {
-                #$DB::single = 1;
-                #print "$self $self->{id} $property_name @values\n";
-            }
-            $value = $value + 0;
-            if ($value !~ /^(\+|\-)?[0-9]*$/)
-            {
-                push @tags, UR::Object::Tag->create
-                (
-                    type => 'invalid',
-                    properties => [$property_name],
-                    desc => 'Invalid integer.'
-                );
-            }
-            # Cleanup for size check below.
-            $value =~ s/[\+\-]//g;
         }
         elsif ($generic_data_type eq 'DateTime') {
             # This check is currently disabled b/c of time format irrecularities
@@ -266,7 +283,8 @@ sub __errors__ {
                 }
             }
             unless ($valid) {
-                my $value_list = join(', ',@$constraints);
+                # undef is a valid value in the constraints list
+                my $value_list = join(', ',map { defined($_) ? $_ : '' } @$constraints);
                 push @tags,
                     UR::Object::Tag->create(
                         type => 'invalid',
@@ -308,24 +326,33 @@ sub __errors__ {
 #  mock objects
 
 sub define_boolexpr {
-    return UR::BoolExpr->resolve(@_);
+    if (ref($_[0])) {
+        my $class = ref(shift);
+        return UR::BoolExpr->resolve($class,@_);
+    }
+    else {
+        return UR::BoolExpr->resolve(@_);
+    }
 }
 
 sub define_set {
     my $class = shift;
     $class = ref($class) || $class;
-    my $rule = UR::BoolExpr->resolve($class,@_);
+    my $rule = UR::BoolExpr->resolve_normalized($class,@_);
+    my $flattened_rule = $rule->flatten_hard_refs();
     my $set_class = $class . "::Set";
-    return $set_class->get($rule->id);
+    return $set_class->get($flattened_rule->id);
 }
 
 sub add_observer {
     my $self = shift;
     my %params = @_;
 
+    if (ref($self)) {
+        $params{subject_id} = $self->id;
+    }
     my $observer = UR::Observer->create(
         subject_class_name => $self->class,
-        subject_id => (ref($self) ? $self->id : undef),
         %params,
     );
     unless ($observer) {
@@ -423,7 +450,7 @@ sub create_mock {
     for my $class_object ($subject_class_object,$subject_class_object->ancestry_class_metas) {
         for my $property ($class_object->direct_property_metas) {
             my $property_name = $property->property_name;
-            if ($property->is_delegated && !exists($params{$property_name})) {
+            if (($property->is_delegated || $property->is_optional) && !exists($params{$property_name})) {
                 next;
             }
             if ($property->is_mutable || $property->is_calculated || $property->is_delegated) {
@@ -484,15 +511,11 @@ sub __changes__ {
     # This is really only useful internally because the boundary of the object
     # is internal/subjective.
 
-    my ($self,$optional_property) = @_;
+    my $self = shift;
 
     # performance optimization
     return unless $self->{_change_count};
 
-    unless (wantarray) {
-        return $self->{_change_count};  # scalar context only cares if there are any changes or not
-    }
-
     my $meta = $self->__meta__;
     if (ref($meta) eq 'UR::DeletedRef') {
         print Data::Dumper::Dumper($self,$meta);
@@ -504,19 +527,43 @@ sub __changes__ {
 
     my $orig = $self->{db_saved_uncommitted} || $self->{db_committed};
 
+    my %prop_metas;
+    my $prop_is_changed = sub {
+        my $prop_name = shift;
+        my $property_meta = $prop_metas{$prop_name} ||= $meta->property_meta_for_name($prop_name);
+        no warnings 'uninitialized';
+        return ($orig->{$prop_name} ne $self->{$prop_name})
+                &&
+                ($self->can($prop_name) and ! UR::Object->can($prop_name))
+                &&
+                defined($property_meta)
+                &&
+               (! $property_meta->is_transient)
+            ;
+    };
+
+    unless (wantarray) {
+        # scalar context only cares if there are any changes or not
+        if (@_) {
+            foreach (@_) {
+                return 1 if $prop_is_changed->($_);
+            }
+            return '';
+        } else {
+            return ($self->{__defined} and $self->{_change_count} == 1)
+                    ? ''
+                    : $self->{_change_count};
+        }
+    }
+
     no warnings;
     my @changed;
     if ($orig) {
         my $class_name = $meta->class_name;
         @changed =
-            grep {
-                my $property_meta = $meta->property_meta_for_name($_);
-                ( ((!$property_meta) or $property_meta->is_transient) ? 0 : 1 );
-            }
-            grep { $self->can($_) and not UR::Object->can($_) }
-            grep { $orig->{$_} ne $self->{$_} }
+            grep { $prop_is_changed->($_) }
             grep { $_ }
-            keys %$orig;
+            @_ ? (@_) : keys(%$orig);
     }
     else {
         @changed = $meta->all_property_names
@@ -608,6 +655,11 @@ sub __extend_namespace__ {
 
 # Handling of references within the current process
 
+sub is_weakened {
+    my $self = shift;
+    return (exists $self->{__weakened} && $self->{__weakened});
+}
+
 sub __weaken__ {
     # Mark this object as unloadable by the object cache pruner.
     # If the class has a data source, then a weakened object is dropped
@@ -620,31 +672,114 @@ sub __weaken__ {
     $self->{'__weakened'} = 1;
 }
 
+sub is_strengthened {
+    my $self = shift;
+    return (exists $self->{__strengthened} && $self->{__strengthened});
+}
+
 sub __strengthen__ {
     # Indicate this object should never be unloaded by the object cache pruner
+    # or AutoUnloadPool
     my $self = $_[0];
     delete $self->{'__weakened'};
     $self->{'__strengthened'} = 1;
 }
 
+sub is_prunable {
+    my $self = shift;
+    return 0 if $self->is_strengthened;
+    return 1 if $self->is_weakened;
+    return 0 if $self->__meta__->is_meta;
+    return 0 if $self->{__get_serial} && $self->__changes__ && @{[$self->__changes__]};
+    return 1;
+}
+
+
+sub __rollback__ {
+    my $self = shift;
+
+    my $saved = $self->{db_saved_uncommitted} || $self->{db_committed};
+    unless ($saved) {
+        return UR::Object::delete($self);
+    }
+
+    my $meta = $self->__meta__;
+
+    my $should_rollback = sub {
+        my $property_meta = shift;
+        return ! (
+            defined $property_meta->is_id
+            || ! defined $property_meta->column_name
+            || $property_meta->is_delegated
+            || $property_meta->is_legacy_eav
+            || ! $property_meta->is_mutable
+            || $property_meta->is_transient
+            || $property_meta->is_constant
+        );
+    };
+    my @rollback_property_names =
+        map { $_->property_name }
+        grep { $should_rollback->($_) }
+        map { $meta->property_meta_for_name($_) }
+        $meta->all_property_names;
+
+    # Existing object.  Undo all changes since last sync, or since load
+    # occurred when there have been no syncs.
+    foreach my $property_name ( @rollback_property_names ) {
+        $self->__rollback_property__($property_name);
+    }
+
+    delete $self->{'_change_count'};
+
+    return $self;
+}
+
+
+sub __rollback_property__ {
+    my ($self, $property_name) = @_;
+    my $saved = $self->{db_saved_uncommitted} || $self->{db_committed};
+    unless ($saved) {
+        Carp::croak(qq(Cannot rollback property '$property_name' because it has no saved state));
+    }
+    my $saved_value = UR::Context->current->value_for_object_property_in_underlying_context($self, $property_name);
+    return $self->$property_name($saved_value);
+}
+
+
 sub DESTROY {
     # Handle weak references in the object cache.
     my $obj = shift;
 
-    # $destroy_should_clean_up_all_objects_loaded will be true if either light_cache is on, or
+    # objects_may_go_out_of_scope will be true if either light_cache is on, or
     # the cache_size_highwater mark is a valid value
-    if ($UR::Context::destroy_should_clean_up_all_objects_loaded) {
-        my $class = ref($obj);
-        my $obj_from_cache = delete $UR::Context::all_objects_loaded->{$class}{$obj->{id}};
+    my($class, $id) = (ref($obj), $obj->{id});
+
+    if (isweak($UR::Context::all_objects_loaded->{$class}{$id})
+        and
+        refaddr($UR::Context::all_objects_loaded->{$class}{$id}) == refaddr($obj)
+    ) {
+        # This object was dropped by the cache pruner or an AutoUnloadPool
+        if (() = $obj->__changes__) {
+            print STDERR "MEM DESTROY keeping changed object $class id $id\n" if $ENV{'UR_DEBUG_OBJECT_RELEASE'};
+            $obj->_save_object_from_destruction();
+            return;
+        } else {
+            print STDERR "MEM DESTROY object $obj class $class if $id\n" if $ENV{'UR_DEBUG_OBJECT_RELEASE'};
+            $obj->unload();
+            return $obj->SUPER::DESTROY();
+        }
+    }
+    elsif (UR::Context::objects_may_go_out_of_scope()) {
+        my $obj_from_cache = delete $UR::Context::all_objects_loaded->{$class}{$id};
         if ($obj->__meta__->is_meta_meta or @{[$obj->__changes__]}) {
-            die "Object found in all_objects_loaded does not match destroyed ref/id! $obj/$obj->{id}!" unless $obj eq $obj_from_cache;
-            $UR::Context::all_objects_loaded->{$class}{$obj->{id}} = $obj;
-            print "KEEPING $obj.  Found $obj .\n";
+            die "Object found in all_objects_loaded does not match destroyed ref/id! $obj/$id!" unless refaddr($obj) == refaddr($obj_from_cache);
+            $obj->_save_object_from_destruction();
+            print "MEM DESTROY Keeping infrastructure/changed object $obj class $class if $id\n" if $ENV{'UR_DEBUG_OBJECT_RELEASE'};
             return;
         }
         else {
             if ($ENV{'UR_DEBUG_OBJECT_RELEASE'}) {
-                print STDERR "MEM DESTROY object $obj class ",$obj->class," id ",$obj->id,"\n";
+                print STDERR "MEM DESTROY object $obj class $class id $id\n";
             }
             $obj->unload();
             return $obj->SUPER::DESTROY();
@@ -652,12 +787,18 @@ sub DESTROY {
     }
     else {
         if ($ENV{'UR_DEBUG_OBJECT_RELEASE'}) {
-            print STDERR "MEM DESTROY object $obj class ",$obj->class," id ",$obj->id,"\n";
+            print STDERR "MEM DESTROY object $obj class $class id $id\n";
         }
         $obj->SUPER::DESTROY();
     }
 };
 
+sub _save_object_from_destruction {
+    my $obj = shift;
+    my($class, $id) = (ref($obj), $obj->{id});
+    $UR::Context::all_objects_loaded->{$class}{$id} = $obj;
+}
+
 END {
     # Turn off monitoring of the DESTROY handler at application exit.
     # setting the typeglob to undef does not work. -sms
@@ -746,7 +887,7 @@ reference represents, using a well-defined interface.
 
 UR uses that separation to automatically handle I/O.  It provides a query API,
 and manages the difference between the state of entities in the application,
-and their state in external persistance systems.  It aims to do so transparently,
+and their state in external persistence systems.  It aims to do so transparently,
 keeping I/O logic orthogonally to "business logic", and hopefully making code
 around I/O unnecessary to write at all for most programs.
 
@@ -891,6 +1032,15 @@ and a fresh reference will later be returnable by get().  See the documentation
 for details on how deleted objects are rememberd and removed later from the database, and how
 deleted objects are re-constructed on STM rollback.
 
+=item copy
+
+  $obj->copy(%overrides)
+
+Copies the existing C<$obj> by copying the values of all direct properties,
+except for ID properties, to a newly created object of the same type.  A list
+of params and values may be provided as overrides to the existing values or to
+specify an ID.
+
 =item class
 
  $class_name = $obj->class;
@@ -1016,7 +1166,7 @@ Add an item to the set of values with add_*:
   $added  = $elmo->add_job($snore);
 
 A variation of the above will construt the item and add it at once.
-This second form of add_* automaticaly would identify that the line items
+This second form of add_* automatically would identify that the line items
 also reference the order, and establish the correct converse relationship
 automatically.
 
@@ -1148,6 +1298,8 @@ Adds an observer to an object, monitoring one or more of its properties for chan
 
 The specified callback is fired upon property changes which match the observation request.
 
+See L<UR::Observer> for details.
+
 =item create_mock
 
  $mock = SomeClass->create_mock(
@@ -1163,7 +1315,7 @@ Useful for test cases.
 
 =head2 Meta API
 
-The folowing methods allow the application to interrogate UR for information
+The following methods allow the application to interrogate UR for information
 about the object in question.
 
 =over 4
@@ -1273,8 +1425,11 @@ Called by all mutators to tell the current context about a state change.
 
   @tags = $obj->__changes__()
 
+  @tags = $obj->__changes__('prop1', 'prop2', ...)
+
 Return a list of changes present on the object _directly_.  This is really only
 useful internally because the boundary of the object is internal/subjective.
+Callers may also request only changes to particular properties.
 
 Changes to objects' properties are tracked by the system.  If an object has been
 changed since it was defined or loaded from its external data source, then changed()
@@ -1306,6 +1461,10 @@ the cache, and will not be considered for pruning.
 
 See L<UR::Context> for more information about the pruning mechanism.
 
+=item is_strengthened
+
+Check if an object has been stengthened, C<__stengthen__>.
+
 =item __weaken__
 
   $obj->__weaken__();
@@ -1314,6 +1473,10 @@ Give a hint to the object cache pruner that this instance is not going to be use
 in the application in the future, and should be removed with preference when
 pruning the cache.
 
+=item is_weakened
+
+Check if an object has been weakened, C<__weaken__>.
+
 =item DESTROY
 
 Perl calls this method on any object before garbage collecting it.  It
