@@ -1,6 +1,6 @@
 package UR::Object::Command::List::Style;
 
-our $VERSION = "0.43"; # UR $VERSION;
+our $VERSION = "0.46"; # UR $VERSION;
 
 sub new {
     my ($class, %args) = @_;
@@ -215,7 +215,7 @@ sub format_and_print{
 
              my $property_node = $object_node->addChild ($doc->createElement($property));
 
-             my @items = $object->$property;
+             my @items = $self->_object_property_to_string($object, $property);
 
              my $reftype = ref $items[0];
 
