@@ -7,7 +7,7 @@ require UR;
 use Lingua::EN::Inflect;
 use Class::AutoloadCAN;
 
-our $VERSION = "0.40"; # UR $VERSION;;
+our $VERSION = "0.46"; # UR $VERSION;;
 our @CARP_NOT = qw( UR::DataSource::RDBMS UR::Object::Type );
 
 # class_meta and r_class_meta duplicate the functionality if two properties of the same name,
@@ -41,6 +41,18 @@ sub is_numeric {
     return $self->{'_is_numeric'};
 }
 
+sub is_text {
+    my $self = shift;
+    unless (defined($self->{'_is_text'})) {
+        my $class = $self->_data_type_as_class_name;
+        unless ($class) {
+            return;
+        }
+        $self->{'_is_text'} = $class->isa("UR::Value::Text");
+    }
+    return $self->{'_is_text'};
+}
+
 sub is_valid_storage_for_value {
     my($self, $value) = @_;
 
@@ -214,7 +226,7 @@ sub _involves_id_property {
 
     if ($self->via) {
         my $via_meta = $self->via_property_meta;
-        return 1 if ($via_meta and $via_meta->_involves_id_property);
+        return 1 if ($via_meta and $via_meta ne $self and $via_meta->_involves_id_property);
 
         if ($self->to) {
             my $to_meta = $self->to_property_meta;
@@ -329,9 +341,13 @@ sub _get_direct_join_linkage {
 
         my @my_id_by = @{ $self->id_by };
         my @their_id_by = @{ $r_class_meta->{'id_by'} };
-        unless (@their_id_by) {
+        if (! @their_id_by
+            or
+            (@my_id_by == 1 and @their_id_by > 1)
+        ) {
             @their_id_by = ( 'id' );
         }
+
         unless (@my_id_by == @their_id_by) {
             Carp::croak("Property '" . $self->property_name . "' of class '" . $self->class_name . "' "
                         . "has " . scalar(@my_id_by) . " id_by elements, while its data_type ("
@@ -353,14 +369,9 @@ sub _get_direct_join_linkage {
 
 sub _resolve_join_chain {
     my $self = shift;
-    my $join_label = shift;
-
-    $join_label ||= $self->property_name;
-
     return UR::Object::Join->resolve_chain(
         $self->class_name,
         $self->property_name,
-        $join_label,
     );
 }
 
@@ -552,7 +563,7 @@ be both constant and mutable
 =item is_volatile => Boolean
 
 Indicates this property can be changed by a mechanism other than its normal
-accessor method.  Signals are not emmitted even when it does change via
+accessor method.  Signals are not emitted even when it does change via
 its normal accessor method.
 
 =item is_classwide => Boolean
@@ -596,7 +607,7 @@ of the property in any way, but is useful in documentation.
 
 =item implied_by => Text
 
-If this propery is created as a result of another property's existence,
+If this property is created as a result of another property's existence,
 implied_by is the name of that other property.  This can happen in the
 case where an object accessor property is defined
 
@@ -629,7 +640,7 @@ get its value from.  The default value is the same as property_name
 
 =item where => ARRAY
 
-Supplies additional filters for indirect properies.  For example:
+Supplies additional filters for indirect properties.  For example:
 
   foos => { is => 'Some::Foo', reverse_as => 'some', is_many => 1 },
   blue_foos => { via => 'foos', where => [ color => 'blue' ] },
