@@ -3,7 +3,7 @@ package UR::Object::View::Default::Xml;
 use strict;
 use warnings;
 require UR;
-our $VERSION = "0.40"; # UR $VERSION;
+our $VERSION = "0.46"; # UR $VERSION;
 use IO::File;
 use XML::Dumper;
 use XML::LibXML;
@@ -13,8 +13,8 @@ class UR::Object::View::Default::Xml {
     has_constant => [
         toolkit     => { value => 'xml' },
     ],
-    has => [
-        _xml_doc    => { is => 'XML::LibXML::Document', doc => 'The LibXML document used to create the content for this view', is_transient => 1 }
+    has_optional_transient => [
+        _xml_doc    => { is => 'XML::LibXML::Document', doc => 'The LibXML document used to create the content for this view', },
     ],
 };
 
