@@ -23,7 +23,7 @@ only change with major releases number increments.
 
 use warnings;
 use strict;
-our $VERSION = "0.40"; # UR $VERSION;
+our $VERSION = "0.46"; # UR $VERSION;
 use Data::Dumper;
 use Scalar::Util qw(blessed);
 
@@ -87,7 +87,7 @@ sub dbh {
                   . "Objects no longer have DB handles, data_sources do\n"
                   . "use resolve_data_sources_for_class_meta_and_rule() on a UR::Context instead");
     my $ds = $UR::Context::current->resolve_data_sources_for_class_meta_and_rule(shift->__meta__);
-    return $ds->get_default_dbh;
+    return $ds->get_default_handle;
 }
 
 
